//
//  IAPHelper.swift
//  selecttv
//
//  Created by Akash Patel on 01/06/20.
//  Copyright © 2020 FreeCast Inc. All rights reserved.
//

import Foundation
import StoreKit
//import SwiftyJSON

class IAPHelper : NSObject, SKProductsRequestDelegate, SKPaymentTransactionObserver {
    
    static let shared = IAPHelper()
    
    var purchaseCompleted : ((_ transaction : SKPaymentTransaction)->Void)?
    var purchaseRestored : ((_ transaction : SKPaymentTransaction)->Void)?
    var purchaseFailed : ((_ errorMessage : String)->Void)?
    var totalRestoredPurchases = 0
    let SUBSCRIPTION_SECRET = ""

    //MARK: -
    
    func startInAppPurchaseWith(productIdentifier : String, iosAppSecret: String, _ completion : @escaping (_ errorMessage : String?, _ transaction : SKPaymentTransaction?, _ transactionID : String?)->Void) {
        if SKPaymentQueue.canMakePayments() {
            //showProgressHUD(withColor: UIColor.progressHUD)
            self.SUBSCRIPTION_SECRET = iosAppSecret
            SKPaymentQueue.default().add(self)
            let productIdentifiers = [productIdentifier]
            let productIdentifierSet = Set.init(productIdentifiers)
            let productRequest = SKProductsRequest.init(productIdentifiers: productIdentifierSet)
            productRequest.delegate = self
            productRequest.start()
            
            self.purchaseFailed = { failedMessage in
                //dismissProgressHUD()
                completion(failedMessage, nil, nil)
            }
            
            self.purchaseCompleted = { transaction in
                self.receiptValidation(isSandbox: false) { (errorMessage, transactionID) in
                    //dismissProgressHUD()
                    if let errorMsg = errorMessage {
                        completion(errorMsg, nil, nil)
                    }
                    else {
                        if transactionID == nil {
                            //Sandbox purchase
                            completion(nil, transaction, transaction.transactionIdentifier)
                        } else {
                            completion(nil, transaction, transactionID)
                        }
                    }
                }
            }
        }
        else {
            completion("Can not perform In-App Purchase", nil, nil)
        }
    }

    func restorePurchase(_ completion : @escaping (_ errorMessage : String?, _ lastTransaction : SKPaymentTransaction?)->Void) {
        if SKPaymentQueue.canMakePayments() {
            //showProgressHUD(withColor: UIColor.progressHUD)
            SKPaymentQueue.default().add(self)
            self.totalRestoredPurchases = 0
            SKPaymentQueue.default().restoreCompletedTransactions()

            self.purchaseFailed = { failedMessage in
                //dismissProgressHUD()
                completion(failedMessage, nil)
            }

            self.purchaseRestored = { transaction in
                //dismissProgressHUD()
                completion(nil, transaction)
            }
        }
        else {
            completion("Can not perform In-App Purchase", nil)
        }
    }

    //MARK: - StoreKit Delegate

    func productsRequest(_ request: SKProductsRequest, didReceive response: SKProductsResponse) {
        if response.products.count > 0 {
            let payment = SKPayment(product: response.products[0])
            SKPaymentQueue.default().add(payment)
        }
        else if response.invalidProductIdentifiers.count != 0 {
            print(response.invalidProductIdentifiers.description)
            self.purchaseFailed?("Can not perform In-App Purchase")
        }
        else {
            self.purchaseFailed?("Can not perform In-App Purchase")
        }
    }

    func paymentQueue(_ queue: SKPaymentQueue, updatedTransactions transactions: [SKPaymentTransaction]) {
        if transactions.count > 0 {
            let transaction = transactions[0]
            switch transaction.transactionState {
            case .purchased:
                print("Transaction completed successfully.")
                SKPaymentQueue.default().finishTransaction(transaction)
                self.purchaseCompleted?(transaction)
                break
            case .failed:
                print("Transaction Failed");
                SKPaymentQueue.default().finishTransaction(transaction)
                self.purchaseFailed?(transaction.error?.localizedDescription ?? "Can not perform In-App Purchase")
                break
            case .purchasing:
                break
            case .restored:
                self.totalRestoredPurchases += 1
                SKPaymentQueue.default().finishTransaction(transaction)
                print("Restored")
                break
            default:
                print(transaction.transactionState.rawValue)
                break
            }
        }
    }

    func paymentQueue(_ queue: SKPaymentQueue, restoreCompletedTransactionsFailedWithError error: Error) {
        if let error = error as? SKError {
            if error.code != .paymentCancelled {
                self.purchaseFailed?(error.localizedDescription)
            } else {
                self.purchaseFailed?("Cancelled In-App Purchase")
            }
        }
    }

    func paymentQueueRestoreCompletedTransactionsFinished(_ queue: SKPaymentQueue) {
        if self.totalRestoredPurchases == 0 {
            self.purchaseFailed?("There is no purchase to restore")
        }
        else {
            if queue.transactions.count > 0 {
                self.purchaseRestored?(queue.transactions[0])
            }
            else {
                self.purchaseFailed?("There is no purchase to restore")
            }
        }
    }

    //MARK: - Verification of AppStore Receipt

    fileprivate func receiptValidation(isSandbox: Bool, _ completion : @escaping (_ error : String?, _ transactionID : String?)->Void) {
        if let receiptURL = Bundle.main.appStoreReceiptURL, FileManager.default.fileExists(atPath: receiptURL.path){
            var receiptData:NSData?
            do{
                receiptData = try NSData(contentsOf: receiptURL, options: NSData.ReadingOptions.alwaysMapped)
            }
            catch{
                print("ERROR: " + error.localizedDescription)
            }
            if let base64encodedReceipt = receiptData?.base64EncodedString(options: NSData.Base64EncodingOptions.endLineWithCarriageReturn) {
                let requestDictionary = ["receipt-data":base64encodedReceipt,"password":self.SUBSCRIPTION_SECRET]

                guard JSONSerialization.isValidJSONObject(requestDictionary) else {  print("requestDictionary is not valid JSON");  return }
                do {
                    let requestData = try JSONSerialization.data(withJSONObject: requestDictionary)
                    let validationURLString = isSandbox ? "https://sandbox.itunes.apple.com/verifyReceipt" : "https://buy.itunes.apple.com/verifyReceipt"  // this works but as noted above it's best to use your own trusted server
                    guard let validationURL = URL(string: validationURLString) else { print("the validation url could not be created, unlikely error"); return }
                    let session = URLSession(configuration: URLSessionConfiguration.default)
                    var request = URLRequest(url: validationURL)
                    request.httpMethod = "POST"
                    request.cachePolicy = URLRequest.CachePolicy.reloadIgnoringCacheData
                    let task = session.uploadTask(with: request, from: requestData) { (data, response, error) in
                        if let data = data , error == nil {
                            do {
                                //let jsonData = JSON.init(data)
                                if let jsonData = try? JSONSerialization.jsonObject(with: data, options: []) as? [String: Any] {
                                    if (jsonData["status"] as? Int)?.description == "21007" {
                                        self.receiptValidation(isSandbox: true, completion)
                                    }
                                    else {
                                    if isSandbox {
                                        completion(nil, nil)
                                    } else {
                                        if let transactionID = (jsonData["latest_receipt_info"] as? [[String : Any]])?.first?["transaction_id"] as? String {
                                            completion(nil, transactionID)
                                        } else {
                                            completion("NA", nil)
                                        }
                                    }
                                    }
                                } else {
                                    completion("Failed to parse JSON", nil)
                                }
                            }
                        } else {
                            print("the upload task returned an error: \(error?.localizedDescription ?? "")")
                            completion(error?.localizedDescription, nil)
                        }
                    }
                    task.resume()
                } catch let error as NSError {
                    print("json serialization failed with error: \(error.localizedDescription)")
                    completion(error.localizedDescription, nil)
                }
            }
        }
    }
}
